#define MY_DEBUG 
#define MY_RADIO_NRF24
#define MY_NODE_ID 7
// Pins for MEGA
#define MY_RF24_CE_PIN 40
#define MY_RF24_CS_PIN 53

//#define MY_REPEATER_FEATURE// Enable repeater functionality for this node

#include <SPI.h>
#include <MySensors.h>

#define cID_DIM_R 0
#define cID_DIM_G 1
#define cID_DIM_B 2
#define cID_FADE_R 3
#define cID_FADE_G 4
#define cID_FADE_B 5
#define cID_CYCLE_RGB 6

bool requestACK = true;

#define SN "LED Strip Controller"
#define SV "0.1"

static int16_t currentLevel = 0;  // Current dim level...
MyMessage msg_DIM_R(cID_DIM_R, V_DIMMER);
MyMessage msg_DIM_G(cID_DIM_G, V_DIMMER);
MyMessage msg_DIM_B(cID_DIM_B, V_DIMMER);

MyMessage msg_SWITCH_R(cID_DIM_R, V_LIGHT);
MyMessage msg_SWITCH_G(cID_DIM_G, V_LIGHT);
MyMessage msg_SWITCH_B(cID_DIM_B, V_LIGHT);

MyMessage msg_FADE_R(cID_FADE_R, V_DIMMER);
MyMessage msg_FADE_G(cID_FADE_G, V_DIMMER);
MyMessage msg_FADE_B(cID_FADE_B, V_DIMMER);

MyMessage msg_SWITCH_FADE_R(cID_FADE_R, V_LIGHT);
MyMessage msg_SWITCH_FADE_G(cID_FADE_G, V_LIGHT);
MyMessage msg_SWITCH_FADE_B(cID_FADE_B, V_LIGHT);

MyMessage msg_CYCLE_RGB(cID_CYCLE_RGB, V_LIGHT);

#include <FastLED.h>
#include <AnalogMatrixKeypad.h>

#define analogPin_1x4 A0
AnalogMatrixKeypad keypad1x4(analogPin_1x4);
unsigned long pMillis_Key = 0;
long sCheck_Key = 200;
int b_count[] = {0,0,0,0}; 
int b_max[] = {2,2,2,1}; 

#define REDPIN   11
#define GREENPIN 9
#define BLUEPIN  10
unsigned long pMillis_LED = 0;
long sCheck_LED = 20;

/*
 * 0 = on/off
 * 1 = brightness 0 to 255
 * 2 = fade effect on/off
 * 3 = fade effect speed 1 to 20
 * 3 = rainbow effect on/off
 */
int LED_R[] = {0,0,0,0};
int LED_G[] = {0,0,0,0};
int LED_B[] = {0,0,0,0};
//int DELAYS[] = {20,100};
int DIRECTION[] = {1,1,1};
int MAX = 255;
int MIN = 0;
int runAnimation = 0;

void setup(){
  Serial.begin(115200);Serial.print("compiled: ");Serial.print(__DATE__);Serial.println(__TIME__);//Base Details
  
  request( cID_DIM_R, V_DIMMER );// Pull the gateway's current dim level - restore light level upon sendor node power-up
  request( cID_DIM_G, V_DIMMER );
  request( cID_DIM_B, V_DIMMER );
  request( cID_FADE_R, V_DIMMER );
  request( cID_FADE_G, V_DIMMER );
  request( cID_FADE_B, V_DIMMER );
  request( cID_CYCLE_RGB, V_LIGHT );
  
  keypad1x4.setDebounceTime(400);
  keypad1x4.setThresholdValue(20);
  keypad1x4.setNumberOfKeys(4);
  
  pinMode(REDPIN,   OUTPUT);
  pinMode(GREENPIN, OUTPUT);
  pinMode(BLUEPIN,  OUTPUT);
  
  colorBars();// Flash the "hello" color sequence: R, G, B, black.
}
void presentation() {
  present( cID_DIM_R, S_DIMMER , "Red LEDs Dimmer", requestACK);
  present( cID_DIM_G, S_DIMMER , "Green LEDs Dimmer", requestACK);
  present( cID_DIM_B, S_DIMMER , "Blue LEDs Dimmer", requestACK);
  
  present( cID_FADE_R, S_DIMMER , "Red LEDs Fade Effect", requestACK);
  present( cID_FADE_G, S_DIMMER , "Green LEDs Fade Effect", requestACK);
  present( cID_FADE_B, S_DIMMER , "Blue LEDs Fade Effect", requestACK);
  
  present( cID_CYCLE_RGB, S_LIGHT , "RGB Cycle LEDs Fade Effect", requestACK);

  sendSketchInfo(SN, SV);
}

void loop(){
  unsigned long currentMillis = millis();
  checkKeyPress(currentMillis);
  updateLEDStrip(currentMillis);
}

void receive(const MyMessage &message) {
  if (message.type == V_LIGHT || message.type == V_DIMMER) {
    int requestedLevel = atoi( message.data );//  Retrieve the power or dim level from the incoming request message
    requestedLevel *= ( message.type == V_LIGHT ? 100 : 1 );// Adjust incoming level if this is a V_LIGHT variable update [0 == off, 1 == on]
    requestedLevel = requestedLevel > 100 ? 100 : requestedLevel;// Clip incoming level to valid range of 0 to 100
    requestedLevel = requestedLevel < 0   ? 0   : requestedLevel;
    int requestedLevel_mapped;
    
    if ((message.sensor == cID_DIM_R) || (message.sensor == cID_DIM_G) || (message.sensor == cID_DIM_B)) {
      requestedLevel_mapped = map(requestedLevel, 0, 100, 0, 255);//map to LED range
    }else if ((message.sensor == cID_FADE_R) || (message.sensor == cID_FADE_G) || (message.sensor == cID_FADE_B)){
      requestedLevel_mapped = map(requestedLevel, 0, 100, 0, 20);//map to Step range
    }
    #ifdef MY_DEBUG
    Serial.print( "Changing level to " );
    Serial.print( requestedLevel );
    Serial.print( " (" );
    Serial.print( requestedLevel_mapped );
    Serial.print( ") , from " );
    Serial.println( currentLevel );
    #endif

    if (message.sensor == cID_DIM_R) {
      #ifdef MY_DEBUG
      Serial.println ("Sending RED Dimmer data");
      #endif
      currentLevel = requestedLevel;        
      LED_R[0] = currentLevel == 0 ? 0 : 1;
      LED_R[1] = requestedLevel_mapped;
      send(msg_SWITCH_R.set(currentLevel > 0));// Inform the gateway of the current DimmableLED's SwitchPower1 and LoadLevelStatus value...
      send( msg_DIM_R.set(currentLevel) );// hek comment: Is this really nessesary?
    }else if (message.sensor == cID_DIM_G) {
      #ifdef MY_DEBUG
      Serial.println ("Sending GREEN Dimmer data");
      #endif
      currentLevel = requestedLevel;
      LED_G[0] = currentLevel == 0 ? 0 : 1;
      LED_G[1] = requestedLevel_mapped;
      send(msg_SWITCH_G.set(currentLevel > 0));
      send( msg_DIM_G.set(currentLevel) );
    }else if (message.sensor == cID_DIM_B) {
      #ifdef MY_DEBUG
      Serial.println ("Sending BLUE Dimmer data");
      #endif
      currentLevel = requestedLevel;
      LED_B[0] = currentLevel == 0 ? 0 : 1;
      LED_B[1] = requestedLevel_mapped;
      send(msg_SWITCH_B.set(currentLevel > 0));
      send( msg_DIM_B.set(currentLevel) );
    }else if (message.sensor == cID_FADE_R) {
      #ifdef MY_DEBUG
      Serial.println ("Sending RED Fade Effect data");
      #endif
      currentLevel = requestedLevel;
      LED_R[2] = currentLevel == 0 ? 0 : 1;
      LED_R[3] = requestedLevel_mapped;
      send(msg_SWITCH_FADE_R.set(currentLevel > 0));
      send(msg_FADE_R.set(currentLevel > 0));
    }else if (message.sensor == cID_FADE_G) {
      #ifdef MY_DEBUG
      Serial.println ("Sending GREEN Fade Effect data");
      #endif
      currentLevel = requestedLevel;
      LED_G[2] = currentLevel == 0 ? 0 : 1;
      LED_G[3] = requestedLevel_mapped;
      send(msg_SWITCH_FADE_G.set(currentLevel > 0));
      send(msg_FADE_G.set(currentLevel > 0));
    }else if (message.sensor == cID_FADE_B) {
      #ifdef MY_DEBUG
      Serial.println ("Sending BLUE Fade Effect data");
      #endif
      currentLevel = requestedLevel;
      LED_B[2] = currentLevel == 0 ? 0 : 1;
      LED_B[3] = requestedLevel_mapped;
      send(msg_SWITCH_FADE_B.set(currentLevel > 0));
      send(msg_FADE_B.set(currentLevel > 0));
    }else if (message.sensor == cID_CYCLE_RGB) {
      #ifdef MY_DEBUG
      Serial.println ("Sending RGB Cycle Effect data");
      #endif
      runAnimation = currentLevel == 0 ? 0 : 1;
      currentLevel = requestedLevel;
      send(msg_CYCLE_RGB.set(currentLevel > 0));
      
    }
  }
}
int returnNextBrightness(int current_brightness, int step_level, int array_Index){
  int sval = 0;
  if (DIRECTION[array_Index] == 1){
    sval = current_brightness + step_level;
    if (sval >= MAX){
      sval = MAX;
      DIRECTION[array_Index] = 0;
    }
  }else if (DIRECTION[array_Index] == 0){
    sval = current_brightness - step_level;
    if (sval <= MIN){
      sval = MIN;
      DIRECTION[array_Index] = 1;
    }
  }
  return sval;
}
void updateLEDStrip(unsigned long theMills){
  if (theMills - pMillis_LED >= sCheck_LED){

    if (runAnimation== 1) {
      static uint8_t hue;
      hue = hue + 1;
      showAnalogRGB( CHSV( hue, 255, 255) );// Use FastLED automatic HSV->RGB conversion
    } else {
      if ((LED_R[0] == 1) && (LED_R[2] == 1))
        LED_R[1] = returnNextBrightness(LED_R[1], LED_R[3], 0);
  
      if (LED_R[0] == 1)
        showAnalogRGB('r', LED_R[1]);
      else if (LED_R[0] == 0)
        showAnalogRGB('r', 0);
        
      if ((LED_G[0] == 1) && (LED_G[2] == 1))
        LED_G[1] = returnNextBrightness(LED_G[1], LED_G[3], 1);
  
      if (LED_G[0] == 1)
        showAnalogRGB('g', LED_G[1]);
      else if (LED_G[0] == 0)
        showAnalogRGB('g', 0);
        
      if ((LED_B[0] == 1) && (LED_B[2] == 1))
        LED_B[1] = returnNextBrightness(LED_B[1], LED_B[3], 2);
  
      if (LED_B[0] == 1)
        showAnalogRGB('b', LED_B[1]);
      else if (LED_B[0] == 0)
        showAnalogRGB('b', 0);
    }
    
    
    pMillis_LED = theMills;
  }
}
void showAnalogRGB( const CRGB& rgb){
  analogWrite(REDPIN,   rgb.r );
  analogWrite(GREENPIN, rgb.g );
  analogWrite(BLUEPIN,  rgb.b );
}
void showAnalogRGB(int r, int g, int b){
  analogWrite(REDPIN,   r );
  analogWrite(GREENPIN, g );
  analogWrite(BLUEPIN,  b );
}
void showAnalogRGB(char c, int ll){
  if (c == 'r')
    analogWrite(REDPIN,   ll );
  else if (c == 'g')
    analogWrite(GREENPIN, ll );
  else if (c == 'b')
    analogWrite(BLUEPIN,  ll );
}
void colorBars(){
  showAnalogRGB( CRGB::Red );   delay(500);
  showAnalogRGB( CRGB::Green ); delay(500);
  showAnalogRGB( CRGB::Blue );  delay(500);
  showAnalogRGB( CRGB::Black ); delay(500);
}

void checkKeyPress(unsigned long theMills){
  if (theMills - pMillis_Key >= sCheck_Key){
    char Key1x4 = keypad1x4.readKey();
      //Serial.println(analogRead(A0));
    int currKey = -1;
    if(Key1x4 != KEY_NOT_PRESSED){
      //Serial.println(Key1x4);
      switch (Key1x4){
        case '1':
          currKey = 1;
          break;
        case '2':
          currKey = 2;
          break;
        case '3':
          currKey = 3;
          break;
        case '4':
          currKey = 4;
          break;
      }
      for (int k=0;k<4;k++){
        if (k == (currKey-1)){
          b_count[k] = b_count[k] + 1;
          if (b_count[k] > b_max[k])
            b_count[k]=0;
          actionKeyPress(k, b_count[k]);
        }else{
          b_count[k] = 0;
        }
      }
    }
    pMillis_Key = theMills;
  }
}
void actionKeyPress(int keyNum, int keyValue){
  int temp_swith_state = 0;
  int temp_swith_fade_state = 0;
  int temp_brightness = 0;
  int temp_steps = 0;
  switch (keyValue){
    case 0: //off
    /*
      temp_swith_state = 0;
      temp_swith_fade_state = 0;
      temp_brightness = 0;
      temp_steps = 0;
      */
      break;
    case 1: //on
      temp_swith_state = 1;
      temp_swith_fade_state = 0;
      temp_brightness = 100;
      temp_steps = 0;
      break;
    case 2: //fade
      if (keyNum != 3){
        temp_swith_state = 1;
        temp_swith_fade_state = 1;
        temp_brightness = 100;
        temp_steps = 50;
      }
      break;
  }
  switch (keyNum){
    case 0:
      LED_R[0] = temp_swith_state;
      LED_R[1] = temp_brightness == 100 ? 255 : 0;
      LED_R[2] = temp_swith_fade_state;
      LED_R[3] = temp_steps / 10;
      send(msg_SWITCH_R.set(temp_swith_state));
      send( msg_DIM_R.set(temp_brightness) );
      send(msg_SWITCH_FADE_R.set(temp_swith_fade_state));
      send(msg_FADE_R.set(temp_steps));
      break;
    case 1:
      LED_G[0] = temp_swith_state;
      LED_G[1] = temp_brightness == 100 ? 255 : 0;
      LED_G[2] = temp_swith_fade_state;
      LED_G[3] = temp_steps / 10;
      send(msg_SWITCH_G.set(temp_swith_state));
      send( msg_DIM_G.set(temp_brightness) );
      send(msg_SWITCH_FADE_G.set(temp_swith_fade_state));
      send(msg_FADE_G.set(temp_steps));
      break;
    case 2:
      LED_B[0] = temp_swith_state;
      LED_B[1] = temp_brightness == 100 ? 255 : 0;
      LED_B[2] = temp_swith_fade_state;
      LED_B[3] = temp_steps / 10;
      send(msg_SWITCH_B.set(temp_swith_state));
      send( msg_DIM_B.set(temp_brightness) );
      send(msg_SWITCH_FADE_B.set(temp_swith_fade_state));
      send(msg_FADE_B.set(temp_steps));
      break;
    case 3:
      runAnimation = temp_swith_state == 1 ? 1 : 0;
      send(msg_CYCLE_RGB.set(runAnimation));
      break;
  }
}

